using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Configuration;
using System.Xml;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;


namespace SessionBrowser
{
    public partial class BrowserForm : Form
    {
        private static readonly string m_sSessionDBConnection = ConfigurationManager.AppSettings["SessionDBConnection"];

        public BrowserForm()
        {
            InitializeComponent();
        }

        private void BrowserForm_Load(object sender, EventArgs e)
        {
            try
            {
                UpdateItem.Enabled = false;
                DBConnection.Text = m_sSessionDBConnection;
                StartDate.Text = DateTime.Now.ToString("MM/dd/yyyy");
                EndDate.Text = DateTime.Now.ToString("MM/dd/yyyy");
                SelectSessions_Click(null, null);
            }
            catch (Exception err)
            {
                ErrMsg.Text = "Error: " + err.Message;
            }
        }

        private void FixInput()
        {
            DBConnection.Text = DBConnection.Text.Trim();
            SessionID.Text = SessionID.Text.Trim();
            ItemKey.Text = ItemKey.Text.Trim();
            StartDate.Text = StartDate.Text.Trim();
            EndDate.Text = EndDate.Text.Trim();
            SelectedItem.Text = "";
            DateTime ed = Convert.ToDateTime(EndDate.Text);
            if (ed.Hour == 0 && ed.Minute == 0 && ed.Second == 0 && ed.Millisecond == 0)
            {
                ed = ed.AddDays(1).AddMilliseconds(-1);
                EndDate.Text = ed.ToString("MM/dd/yyyy HH:mm:ss");
            }
        }

        private void ClearLists()
        {
            SessionList.Items.Clear();
            ItemList.Items.Clear();
            SelectedItem.Text = "";
        }

        private void SelectSessions_Click(object sender, EventArgs e)
        {
            try
            {
                ErrMsg.Text = "";
                FixInput();
                ClearLists();
                UpdateItem.Enabled = false;
                DataSet ds = FindSessions(SessionID.Text, StartDate.Text, EndDate.Text);
                FillSessionList(ds);
            }
            catch (Exception err)
            {
                ErrMsg.Text = "Error: " + err.Message;
            }
        }

        private DataSet FindSessions(string sSessionID, string sStartDate, string sEndDate)
        {
            string sSQL = "exec dbo.BrowseSessions N'" + sSessionID.Replace("'", "''") + "', null, '" + sStartDate.Replace("'", "''") + "', '" + sEndDate.Replace("'", "''") + "'";
            DataSet ds = SessionService.Session.ExecuteSQL(DBConnection.Text, sSQL);
            return ds;
        }

        private void FillSessionList(DataSet ds)
        {
            if (ds.Tables.Count > 0 && ds.Tables[0].Rows.Count > 0)
            {
                for (int i = 0; i < ds.Tables[0].Rows.Count; i++)
                {
                    SessionList.Items.Add(new ListViewItem(new string[] { ds.Tables[0].Rows[i]["SessionID"].ToString(), ds.Tables[0].Rows[i]["LastAccess"].ToString(), ds.Tables[0].Rows[i]["DateTimeCreated"].ToString(), ds.Tables[0].Rows[i]["Timeout"].ToString()}));    
                }
            }
        }

        private void Clear_Click(object sender, EventArgs e)
        {
            ClearLists();
            SessionID.Text = "";
            ItemKey.Text = "";
            StartDate.Text = "";
            EndDate.Text = "";
            SelectedItem.Text = "";
        }

        private void SessionList_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                ErrMsg.Text = "";
                SelectedItem.Text = "";
                ItemList.Items.Clear();
                UpdateItem.Enabled = false;
                if (SessionList.SelectedIndices.Count == 0) return;
                int index = SessionList.SelectedIndices[0];
                string sSessionID = SessionList.Items[index].SubItems[0].Text;
                DataSet ds = FindItems(sSessionID, "");
                FillItemList(ds);
            }
            catch (Exception err)
            {
                ErrMsg.Text = "Error: " + err.Message;
            }
        }

        private DataSet FindItems(string sSessionID, string sItemKey)
        {
            string sSQL = "exec dbo.BrowseSessions N'" + sSessionID.Replace("'", "''") + "', N'" + sItemKey.Replace("'", "''") + "'";
            return SessionService.Session.ExecuteSQL(DBConnection.Text, sSQL);
        }

        private void FillItemList(DataSet ds)
        {
            if (ds.Tables.Count > 0 && ds.Tables[0].Rows.Count > 0)
            {
                for (int i = 0; i < ds.Tables[0].Rows.Count; i++)
                {
                    ItemList.Items.Add(new ListViewItem(new string[] { ds.Tables[0].Rows[i]["SessionID"].ToString(), ds.Tables[0].Rows[i]["ItemKey"].ToString(), ds.Tables[0].Rows[i]["ItemText"].ToString()}));
                }
            }
        }

        private void ItemList_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                ErrMsg.Text = "";
                SelectedItem.Text = "";
                UpdateItem.Enabled = false;
                if (ItemList.SelectedIndices.Count == 0) return;
                int index = ItemList.SelectedIndices[0];
                string sSessionID = ItemList.Items[index].SubItems[0].Text;
                string sItemKey = ItemList.Items[index].SubItems[1].Text;
                string sSQL = "exec dbo.BrowseSessions N'" + sSessionID.Replace("'", "''") + "', N'" + sItemKey.Replace("'", "''") + "'";
                DataSet ds = SessionService.Session.ExecuteSQL(DBConnection.Text, sSQL);
                if (ds.Tables.Count > 0 && ds.Tables[0].Rows.Count > 0)
                {
                    string sItemText = ds.Tables[0].Rows[0]["ItemText"].ToString();
                    if (sItemText != null && sItemKey != "")
                    {
                        byte[] data = Convert.FromBase64String(sItemText);
                        if (data != null && data.Length > 0)
                        {
                            BinaryFormatter formatter = new BinaryFormatter();
                            MemoryStream stream = new MemoryStream(data);
                            object obj = formatter.Deserialize(stream);
                            if (obj != null)
                            {
                                XmlDocument doc = new XmlDocument();
                                doc.LoadXml("<?xml version=\"1.0\" encoding=\"UTF-8\"?><SessionItemRoot><SessionID></SessionID><ItemKey></ItemKey><SessionItem></SessionItem></SessionItemRoot>");
                                doc.SelectSingleNode("//SessionItemRoot/SessionID").InnerText = sSessionID;
                                doc.SelectSingleNode("//SessionItemRoot/ItemKey").InnerText = sItemKey;
                                XmlNode root = doc.SelectSingleNode("//SessionItemRoot/SessionItem");
                                BuildXML(ref root, obj);
                                SelectedItem.Text = doc.OuterXml;
                            }
                        }
                    }
                    UpdateItem.Enabled = true;
                }
            }
            catch (Exception err)
            {
                ErrMsg.Text = "Error: " + err.Message;
            }
        }

        private void BuildXML(ref XmlNode node, object obj)
        {
            string sType = obj.GetType().ToString();
            switch (sType)
            {
                case "System.Object":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Object");
                        XmlNode newnode = node.AppendChild(element);
                        if (obj == null)
                        {
                            XmlAttribute att = node.OwnerDocument.CreateAttribute("IsNull");
                            newnode.Attributes.Append(att);
                            att.InnerText = "true";
                        }
                        break;
                    }
                case "System.String":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.String");
                        XmlNode newnode = node.AppendChild(element);
                        if (obj == null)
                        {
                            XmlAttribute att = node.OwnerDocument.CreateAttribute("IsNull");
                            newnode.Attributes.Append(att);
                            att.InnerText = "true";
                        }
                        else newnode.InnerText = obj.ToString();
                        break;
                    }
                case "System.Boolean":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Boolean");
                        XmlNode newnode = node.AppendChild(element);
                        newnode.InnerText = obj.ToString();
                        break;
                    }
                case "System.Byte":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Byte");
                        XmlNode newnode = node.AppendChild(element);
                        newnode.InnerText = obj.ToString();
                        break;
                    }
                case "System.Char":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Char");
                        XmlNode newnode = node.AppendChild(element);
                        newnode.InnerText = obj.ToString();
                        break;
                    }
                case "System.Int16":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Int16");
                        XmlNode newnode = node.AppendChild(element);
                        newnode.InnerText = obj.ToString();
                        break;
                    }
                case "System.Int32":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Int32");
                        XmlNode newnode = node.AppendChild(element);
                        newnode.InnerText = obj.ToString();
                        break;
                    }
                case "System.Int64":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Int64");
                        XmlNode newnode = node.AppendChild(element);
                        newnode.InnerText = obj.ToString();
                        break;
                    }
                case "System.Double":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Double");
                        XmlNode newnode = node.AppendChild(element);
                        newnode.InnerText = obj.ToString();
                        break;
                    }
                case "System.Decimal":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Decimal");
                        XmlNode newnode = node.AppendChild(element);
                        newnode.InnerText = obj.ToString();
                        break;
                    }
                case "System.DateTime":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.DateTime");
                        XmlNode newnode = node.AppendChild(element);
                        newnode.InnerText = obj.ToString();
                        break;
                    }
                case "System.Object[]":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Object.Array");
                        XmlNode newnode = node.AppendChild(element);
                        if (obj == null)
                        {
                            XmlAttribute att = node.OwnerDocument.CreateAttribute("IsNull");
                            newnode.Attributes.Append(att);
                            att.InnerText = "true";
                        }
                        else
                        {
                            object[] array = (object[])obj;
                            for (int i = 0; i < array.Length; i++)
                            {
                                if (array[i] == null)
                                {
                                    XmlElement element2 = node.OwnerDocument.CreateElement("System.Object");
                                    XmlNode newnode2 = newnode.AppendChild(element2);
                                    XmlAttribute att = node.OwnerDocument.CreateAttribute("IsNull");
                                    newnode2.Attributes.Append(att);
                                    att.InnerText = "true";
                                }
                                else BuildXML(ref newnode, array[i]);
                            }
                        }
                        break;
                    }
                case "System.String[]":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.String.Array");
                        XmlNode newnode = node.AppendChild(element);
                        if (obj == null)
                        {
                            XmlAttribute att = node.OwnerDocument.CreateAttribute("IsNull");
                            newnode.Attributes.Append(att);
                            att.InnerText = "true";
                        }
                        else
                        {
                            string[] array = (string[])obj;
                            for (int i = 0; i < array.Length; i++)
                            {
                                if (array[i] == null)
                                {
                                    XmlElement element2 = node.OwnerDocument.CreateElement("System.String");
                                    XmlNode newnode2 = newnode.AppendChild(element2);
                                    XmlAttribute att = node.OwnerDocument.CreateAttribute("IsNull");
                                    newnode2.Attributes.Append(att);
                                    att.InnerText = "true";
                                }
                                else BuildXML(ref newnode, array[i]);
                            }
                        }
                        break;
                    }
                case "System.Boolean[]":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Boolean.Array");
                        XmlNode newnode = node.AppendChild(element);
                        if (obj == null)
                        {
                            XmlAttribute att = node.OwnerDocument.CreateAttribute("IsNull");
                            newnode.Attributes.Append(att);
                            att.InnerText = "true";
                        }
                        else
                        {
                            bool[] array = (bool[])obj;
                            for (int i = 0; i < array.Length; i++)
                            {
                                BuildXML(ref newnode, array[i]);
                            }
                        }
                        break;
                    }
                case "System.Byte[]":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Byte.Array");
                        XmlNode newnode = node.AppendChild(element);
                        if (obj == null)
                        {
                            XmlAttribute att = node.OwnerDocument.CreateAttribute("IsNull");
                            newnode.Attributes.Append(att);
                            att.InnerText = "true";
                        }
                        else
                        {
                            byte[] array = (byte[])obj;
                            for (int i = 0; i < array.Length; i++)
                            {
                                BuildXML(ref newnode, array[i]);
                            }
                        }
                        break;
                    }
                case "System.Char[]":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Char.Array");
                        XmlNode newnode = node.AppendChild(element);
                        if (obj == null)
                        {
                            XmlAttribute att = node.OwnerDocument.CreateAttribute("IsNull");
                            newnode.Attributes.Append(att);
                            att.InnerText = "true";
                        }
                        else
                        {
                            char[] array = (char[])obj;
                            for (int i = 0; i < array.Length; i++)
                            {
                                BuildXML(ref newnode, array[i]);
                            }
                        }
                        break;
                    }
                case "System.Int16[]":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Int16.Array");
                        XmlNode newnode = node.AppendChild(element);
                        if (obj == null)
                        {
                            XmlAttribute att = node.OwnerDocument.CreateAttribute("IsNull");
                            newnode.Attributes.Append(att);
                            att.InnerText = "true";
                        }
                        else
                        {
                            short[] array = (short[])obj;
                            for (int i = 0; i < array.Length; i++)
                            {
                                BuildXML(ref newnode, array[i]);
                            }
                        }
                        break;
                    }
                case "System.Int32[]":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Int32.Array");
                        XmlNode newnode = node.AppendChild(element);
                        if (obj == null)
                        {
                            XmlAttribute att = node.OwnerDocument.CreateAttribute("IsNull");
                            newnode.Attributes.Append(att);
                            att.InnerText = "true";
                        }
                        else
                        {
                            int[] array = (int[])obj;
                            for (int i = 0; i < array.Length; i++)
                            {
                                BuildXML(ref newnode, array[i]);
                            }
                        }
                        break;
                    }
                case "System.Int64[]":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Int64.Array");
                        XmlNode newnode = node.AppendChild(element);
                        if (obj == null)
                        {
                            XmlAttribute att = node.OwnerDocument.CreateAttribute("IsNull");
                            newnode.Attributes.Append(att);
                            att.InnerText = "true";
                        }
                        else
                        {
                            long[] array = (long[])obj;
                            for (int i = 0; i < array.Length; i++)
                            {
                                BuildXML(ref newnode, array[i]);
                            }
                        }
                        break;
                    }
                case "System.Double[]":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Double.Array");
                        XmlNode newnode = node.AppendChild(element);
                        if (obj == null)
                        {
                            XmlAttribute att = node.OwnerDocument.CreateAttribute("IsNull");
                            newnode.Attributes.Append(att);
                            att.InnerText = "true";
                        }
                        else
                        {
                            double[] array = (double[])obj;
                            for (int i = 0; i < array.Length; i++)
                            {
                                BuildXML(ref newnode, array[i]);
                            }
                        }
                        break;
                    }
                case "System.Decimal[]":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.Decimal.Array");
                        XmlNode newnode = node.AppendChild(element);
                        if (obj == null)
                        {
                            XmlAttribute att = node.OwnerDocument.CreateAttribute("IsNull");
                            newnode.Attributes.Append(att);
                            att.InnerText = "true";
                        }
                        else
                        {
                            decimal[] array = (decimal[])obj;
                            for (int i = 0; i < array.Length; i++)
                            {
                                BuildXML(ref newnode, array[i]);
                            }
                        }
                        break;
                    }
                case "System.DateTime[]":
                    {
                        XmlElement element = node.OwnerDocument.CreateElement("System.DateTime.Array");
                        XmlNode newnode = node.AppendChild(element);
                        if (obj == null)
                        {
                            XmlAttribute att = node.OwnerDocument.CreateAttribute("IsNull");
                            newnode.Attributes.Append(att);
                            att.InnerText = "true";
                        }
                        else
                        {
                            DateTime[] array = (DateTime[])obj;
                            for (int i = 0; i < array.Length; i++)
                            {
                                BuildXML(ref newnode, array[i]);
                            }
                        }
                        break;
                    }
                default:
                    throw new Exception("Cannot handle this type: " + sType);
                
            }
        }

        private void SelectItems_Click(object sender, EventArgs e)
        {
            try
            {
                ErrMsg.Text = "";
                ItemList.Items.Clear();
                UpdateItem.Enabled = false;
                SessionID.Text = SessionID.Text.Trim();
                ItemKey.Text = ItemKey.Text.Trim();
                DataSet ds = FindItems(SessionID.Text, ItemKey.Text);
                FillItemList(ds);
            }
            catch (Exception err)
            {
                ErrMsg.Text = "Error: " + err.Message;
            }
        }

        private void UpdateItem_Click(object sender, EventArgs e)
        {
            try
            {
                ErrMsg.Text = "";
                if (ItemList.SelectedIndices.Count == 0) throw new Exception("No item in item list selected");
                int index = ItemList.SelectedIndices[0];
                string sSessionID = ItemList.Items[index].SubItems[0].Text;
                string sItemKey = ItemList.Items[index].SubItems[1].Text;
                object obj = new object();
                if (SelectedItem.Text != "")
                {
                    XmlDocument doc = new XmlDocument();
                    doc.LoadXml(SelectedItem.Text);
                    string sSessionID2 = doc.SelectSingleNode("//SessionItemRoot/SessionID").InnerText;
                    string sItemKey2 = doc.SelectSingleNode("//SessionItemRoot/ItemKey").InnerText;
                    if (sSessionID != sSessionID2 || sItemKey != sItemKey2) throw new Exception("Cannot modify session id or item key in xml text");
                    BuildObject(doc.SelectSingleNode("//SessionItemRoot/SessionItem").ChildNodes[0], ref obj);
                    SessionService.Session.SetSessionData(sSessionID, sItemKey, obj);
                    SelectedItem.Text = "";
                    SelectSessions_Click(null, null);
                    if (SessionList.SelectedItems != null && SessionList.SelectedItems.Count > 0) SessionList.SelectedItems[0].Selected = false;
                }
            }
            catch (Exception err)
            {
                ErrMsg.Text = "Error: " + err.Message;
            }
        }

        private void BuildObject(XmlNode node, ref object obj)
        {
            string sType = node.Name;
            switch (sType)
            {
                case "System.Object":
                    {
                        if (node.Attributes.Count > 0 && node.Attributes["IsNull"].InnerText == "true") obj = null;
                        else obj = new object();
                        break;
                    }
                case "System.String":
                    {
                        if (node.Attributes.Count > 0 && node.Attributes["IsNull"].InnerText == "true") obj = null;
                        else obj = node.InnerText;
                        break;
                    }
                case "System.Boolean":
                    {
                        obj = Convert.ToBoolean(node.InnerText);
                        break;
                    }
                case "System.Byte":
                    {
                        obj = Convert.ToByte(node.InnerText);
                        break;
                    }
                case "System.Char":
                    {
                        obj = Convert.ToChar(node.InnerText);
                        break;
                    }
                case "System.Int16":
                    {
                        obj = Convert.ToInt16(node.InnerText);
                        break;
                    }
                case "System.Int32":
                    {
                        obj = Convert.ToInt32(node.InnerText);
                        break;
                    }
                case "System.Int64":
                    {
                        obj = Convert.ToInt64(node.InnerText);
                        break;
                    }
                case "System.Double":
                    {
                        obj = Convert.ToDouble(node.InnerText);
                        break;
                    }
                case "System.Decimal":
                    {
                        obj = Convert.ToDecimal(node.InnerText);
                        break;
                    }
                case "System.DateTime":
                    {
                        obj = Convert.ToDateTime(node.InnerText);
                        break;
                    }
                case "System.Object.Array":
                    {
                        if (node.Attributes.Count > 0 && node.Attributes["IsNull"].InnerText == "true") obj = null;
                        else
                        {
                            object[] array = new object[node.ChildNodes.Count];
                            for (int i = 0; i < array.Length; i++)
                            {
                                if (node.ChildNodes[i].Attributes.Count > 0 && node.ChildNodes[i].Attributes[0].InnerText == "true")
                                    array[i] = null;
                                else BuildObject(node.ChildNodes[i], ref array[i]);
                            }
                            obj = array;
                        }
                        break;
                    }
                case "System.String.Array":
                    {
                        if (node.Attributes.Count > 0 && node.Attributes["IsNull"].InnerText == "true") obj = null;
                        else
                        {
                            string[] array = new string[node.ChildNodes.Count];
                            for (int i = 0; i < array.Length; i++)
                            {
                                if (node.ChildNodes[i].Attributes.Count > 0 && node.ChildNodes[i].Attributes[0].InnerText == "true")
                                    array[i] = null;
                                else array[i] = (node.ChildNodes[i].InnerText);
                            }
                            obj = array;
                        }
                        break;
                    }
                case "System.Boolean.Array":
                    {
                        if (node.Attributes.Count > 0 && node.Attributes["IsNull"].InnerText == "true") obj = null;
                        else
                        {
                            bool[] array = new bool[node.ChildNodes.Count];
                            for (int i = 0; i < array.Length; i++)
                            {
                                array[i] = Convert.ToBoolean(node.ChildNodes[i].InnerText);
                            }
                            obj = array;
                        }
                        break;
                    }
                case "System.Byte.Array":
                    {
                        if (node.Attributes.Count > 0 && node.Attributes["IsNull"].InnerText == "true") obj = null;
                        else
                        {
                            byte[] array = new byte[node.ChildNodes.Count];
                            for (int i = 0; i < array.Length; i++)
                            {
                                array[i] = Convert.ToByte(node.ChildNodes[i].InnerText);
                            }
                            obj = array;
                        }
                        break;
                    }
                case "System.Char.Array":
                    {
                        if (node.Attributes.Count > 0 && node.Attributes["IsNull"].InnerText == "true") obj = null;
                        else
                        {
                            char[] array = new char[node.ChildNodes.Count];
                            for (int i = 0; i < array.Length; i++)
                            {
                                array[i] = Convert.ToChar(node.ChildNodes[i].InnerText);
                            }
                            obj = array;
                        }
                        break;
                    }
                case "System.Int16.Array":
                    {
                        if (node.Attributes.Count > 0 && node.Attributes["IsNull"].InnerText == "true") obj = null;
                        else
                        {
                            short[] array = new short[node.ChildNodes.Count];
                            for (int i = 0; i < array.Length; i++)
                            {
                                array[i] = Convert.ToInt16(node.ChildNodes[i].InnerText);
                            }
                            obj = array;
                        }
                        break;
                    }
                case "System.Int32.Array":
                    {
                        if (node.Attributes.Count > 0 && node.Attributes["IsNull"].InnerText == "true") obj = null;
                        else
                        {
                            int[] array = new int[node.ChildNodes.Count];
                            for (int i = 0; i < array.Length; i++)
                            {
                                array[i] = Convert.ToInt32(node.ChildNodes[i].InnerText);
                            }
                            obj = array;
                        }
                        break;
                    }
                case "System.Int64.Array":
                    {
                        if (node.Attributes.Count > 0 && node.Attributes["IsNull"].InnerText == "true") obj = null;
                        else
                        {
                            long[] array = new long[node.ChildNodes.Count];
                            for (int i = 0; i < array.Length; i++)
                            {
                                array[i] = Convert.ToInt64(node.ChildNodes[i].InnerText);
                            }
                            obj = array;
                        }
                        break;
                    }
                case "System.Double.Array":
                    {
                        if (node.Attributes.Count > 0 && node.Attributes["IsNull"].InnerText == "true") obj = null;
                        else
                        {
                            double[] array = new double[node.ChildNodes.Count];
                            for (int i = 0; i < array.Length; i++)
                            {
                                array[i] = Convert.ToDouble(node.ChildNodes[i].InnerText);
                            }
                            obj = array;
                        }
                        break;
                    }
                case "System.Decimal.Array":
                    {
                        if (node.Attributes.Count > 0 && node.Attributes["IsNull"].InnerText == "true") obj = null;
                        else
                        {
                            decimal[] array = new decimal[node.ChildNodes.Count];
                            for (int i = 0; i < array.Length; i++)
                            {
                                array[i] = Convert.ToDecimal(node.ChildNodes[i].InnerText);
                            }
                            obj = array;
                        }
                        break;
                    }
                case "System.DateTime.Array":
                    {
                        if (node.Attributes.Count > 0 && node.Attributes["IsNull"].InnerText == "true") obj = null;
                        else
                        {
                            DateTime[] array = new DateTime[node.ChildNodes.Count];
                            for (int i = 0; i < array.Length; i++)
                            {
                                array[i] = Convert.ToDateTime(node.ChildNodes[i].InnerText);
                            }
                            obj = array;
                        }
                        break;
                    }
                default:
                    throw new Exception("Cannot handle this type: " + sType);

            }
        }

        private void CreateSession_Click(object sender, EventArgs e)
        {
            try
            {
                ErrMsg.Text = "";
                UpdateItem.Enabled = false;
                SessionID.Text = SessionID.Text.Trim();
                if (SessionID.Text == "") throw new Exception("Session id cannot be empty");
                DateTime now = DateTime.Now;
                DataSet ds = FindSessions(SessionID.Text, Convert.ToDateTime(now.ToString("MM/dd/yyyy")).ToString(), Convert.ToDateTime(now.ToString("MM/dd/yyyy")).AddDays(1).AddMilliseconds(-1).ToString());
                if (ds.Tables.Count > 0 && ds.Tables[0].Rows.Count > 0)
                {
                    for (int i = 0; i < ds.Tables[0].Rows.Count; i++)
                    {
                        if (ds.Tables[0].Rows[i]["SessionID"].ToString() == SessionID.Text)
                            throw new Exception("Session with id '" + SessionID.Text + "' already exists");
                    }
                }
                SessionService.Session.CreateSession(SessionID.Text, Convert.ToInt32(TimeoutValue.Text));
                SelectedItem.Text = "";
                SessionID.Text = "";
                SelectSessions_Click(null, null);
            }
            catch (Exception err)
            {
                ErrMsg.Text = "Error: " + err.Message;
            }
        }

        private void CreateItem_Click(object sender, EventArgs e)
        {
            try
            {
                ErrMsg.Text = "";
                UpdateItem.Enabled = false;
                SessionID.Text = SessionID.Text.Trim();
                ItemKey.Text = ItemKey.Text.Trim();
                if (SessionID.Text == "" || ItemKey.Text == "") throw new Exception("Session id or item key cannot be empty");
                DataSet ds = FindItems(SessionID.Text, ItemKey.Text);
                if (ds.Tables.Count > 0 && ds.Tables[0].Rows.Count > 0)
                {
                    for (int i = 0; i < ds.Tables[0].Rows.Count; i++)
                    {
                        if (ds.Tables[0].Rows[i]["SessionID"].ToString() == SessionID.Text && ds.Tables[0].Rows[i]["ItemKey"].ToString() == ItemKey.Text)
                            throw new Exception("Session item already exists");
                    }
                }
                SessionService.Session.SetSessionData(SessionID.Text, ItemKey.Text, "");
                SelectedItem.Text = "";
                SessionID.Text = "";
                ItemKey.Text = "";
                SelectItems_Click(null, null);
            }
            catch (Exception err)
            {
                ErrMsg.Text = "Error: " + err.Message;
            }
        }

        private static string ObjectToBase64(object obj)
        {
            if (obj == null) return null;
            BinaryFormatter formatter = new BinaryFormatter();
            MemoryStream stream = new MemoryStream(64 * 1024);
            formatter.Serialize(stream, obj);
            byte[] output = stream.ToArray();
            return Convert.ToBase64String(output);
        }

        private void DeleteSession_Click(object sender, EventArgs e)
        {
            try
            {
                ErrMsg.Text = "";
                UpdateItem.Enabled = false;
                SessionID.Text = SessionID.Text.Trim();
                if (SessionID.Text == "") throw new Exception("Session id cannot be empty");
                if (SessionList.SelectedItems.Count == 0 || SessionList.SelectedItems[0].SubItems[0].Text != SessionID.Text)
                    throw new Exception("The session must be selected before deleting");
                SessionService.Session.RemoveSession(SessionID.Text);
                SelectedItem.Text = "";
                SessionID.Text = "";
                SelectSessions_Click(null, null);
            }
            catch (Exception err)
            {
                ErrMsg.Text = "Error: " + err.Message;
            }
        }        
    }
}