using System;
using System.Collections;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Reflection;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Threading;

namespace SessionService
{
	public class Session
	{
		private static readonly string m_sSessionDBConnection = ConfigurationManager.AppSettings["SessionDBConnection"];
        private static readonly int m_nSessionCleanupInterval = ConfigurationManager.AppSettings["SessionCleanupInterval"] == null ? 30 : Convert.ToInt32(ConfigurationManager.AppSettings["SessionCleanupInterval"]);
        private static bool m_bCleanupStarted = false;

        
        private static string ObjectToBase64(object obj)
        {
            if (obj == null) return null;
            BinaryFormatter formatter = new BinaryFormatter();
            MemoryStream stream = new MemoryStream(64*1024);
            formatter.Serialize(stream, obj);
            byte[] output = stream.ToArray();
            return Convert.ToBase64String(output);
        }

        private static object Base64ToObject(string input)
        {
            if (input == null || input == "") return null;
            byte[] data = Convert.FromBase64String(input);
            if (data == null) return null;
            BinaryFormatter formatter = new BinaryFormatter();
            MemoryStream stream = new MemoryStream(data);
            return formatter.Deserialize(stream);
        }

		public static void StartCleanupProcess()
		{
            lock (typeof(Session))
            {
                if(m_bCleanupStarted==false)
                {
                    if (m_sSessionDBConnection == null) throw new Exception("App Setting 'SessionDBConnection' is missing from configuration file");
                    m_bCleanupStarted = true;
                    if (m_nSessionCleanupInterval > 0) (new Thread(new ThreadStart(CleanupOldSessions))).Start();
                }
            }
		}

        private static void CleanupOldSessions()
        {
            if (m_nSessionCleanupInterval <= 0) return;
            while (true)
            {
                try
                {
                    Thread.Sleep(m_nSessionCleanupInterval * 60 * 1000);
                    RemoveSession(null);
                }
                catch (Exception) { }
            }
        }

        public static void CreateSession(string sID, int nTimeout)
        {
            if (sID == null || sID == "") throw new Exception("Invalid session id");
            if (nTimeout < 0) throw new Exception("Invalid timeout value");
            ExecuteSQL(m_sSessionDBConnection, "exec dbo.CreateSession N'" + sID.Replace("'", "''") + "', " + nTimeout.ToString());
        }

		public static void SetSessionTimeout(string sID, int nTimeout)
		{
            if (sID == null || sID == "") throw new Exception("Invalid session id");
            if (nTimeout < 0) throw new Exception("Invalid timeout value");
            ExecuteSQL(m_sSessionDBConnection, "exec dbo.SetSessionTimeout N'" + sID.Replace("'", "''") + "', " + nTimeout.ToString());
		}

		public static void SetSessionData(string sID, string sKey, object oValue)
		{
            if (sID == null || sID == "" || sKey == null || sKey == "") throw new Exception("Invalid session id or key value");
            ExecuteSQL(m_sSessionDBConnection, "exec dbo.SetSessionData N'" + sID.Replace("'", "''") + "', N'" + sKey.Replace("'", "''") + "','" + ObjectToBase64(oValue) + "'");
		}

		public static object GetSessionData(string sID, string sKey)
		{
            if (sID == null || sID == "" || sKey == null || sKey == "") throw new Exception("Invalid session id or key value");
            DataSet ds = ExecuteSQL(m_sSessionDBConnection, "exec dbo.GetSessionData N'" + sID.Replace("'", "''") + "', N'" + sKey.Replace("'", "''") + "'"); 
            if (ds.Tables.Count == 0 || ds.Tables[0].Rows.Count == 0) return null;
            else
            {
                string sBase64 = Convert.ToString(ds.Tables[0].Rows[0]["ItemText"]);
                return Base64ToObject(sBase64);
            }
		}

		public static void RemoveSession(string sID)
		{
            try
            {
                ExecuteSQL(m_sSessionDBConnection, "exec dbo.RemoveSession" + (sID == null ? "" : (" N'" + sID.Replace("'", "''") + "'")));
            }
            catch (Exception) { }
		}

        public static void RemoveSessionData(string sID, string sKey)
        {
            try
            {
                ExecuteSQL(m_sSessionDBConnection, "exec dbo.RemoveSessionData N'" + sID.Replace("'", "''") + "', N'" + sKey.Replace("'", "''") + "'");
            }
            catch (Exception) { }
        }

		public static void SetAppSetting(string sAppName, string sKey, object oValue)
		{
            if (sAppName == null || sAppName == "" || sKey == null || sKey == "") throw new Exception("Invalid session id or key value");
            string sSessionID = "__@@##" + sAppName;
            SetSessionTimeout(sSessionID, 30 * 366 * 24 * 60);
            SetSessionData(sSessionID, sKey, oValue);
		}

		public static object GetAppSetting(string sAppName, string sKey)
		{
            string sSessionID = "__@@##" + sAppName;
            return GetSessionData(sSessionID, sKey);
		}

        public static DataSet ExecuteSQL(string sConnection, string sSQL)
        {
            SqlConnection oConnection = null;
            try
            {
                oConnection = new SqlConnection(sConnection);
                SqlCommand oCommand = oConnection.CreateCommand();
                oCommand.CommandType = CommandType.Text;
                oCommand.CommandText = sSQL;
                SqlDataAdapter oAdapter = new SqlDataAdapter(oCommand);
                oAdapter.TableMappings.Add("Table", "Table0");
                DataSet oDataSet = new DataSet("DataSet");
                oAdapter.Fill(oDataSet);
                return oDataSet;
            }
            catch (Exception)
            {
                throw;
            }
            finally
            {
                try { if (oConnection != null) oConnection.Close(); }
                catch (Exception) { }
            }
        }
	}
}
